#Copyright 2008 Cold Spring Harbor Laboratory.
# Written by Yaniv Erlich (erlich@cshl.edu)

# This package generates all possible permutations for n-arrays.

package Permute;
use strict;
use warnings;
use POSIX qw (floor ceil);

require Exporter;

our @ISA    = ("Exporter");
our @EXPORT = qw(permute);
our $VERSION = 1.00;



sub permute {

    #Input:   Array that cotains refrences to n arrays that
    #         can be in different lenghts
    #Output:  Return all the possible permutaitions of the items
    #         in the arrays. For example: @a=(1,2,3) and @b=(7,8):
    #         gives: (1,7),(1,8),(2,7),(2,8),(3,7),(3,8). Maybe not in the order

    #Process: The idea is first to create a numberical system that 
    #         its base of the i-th digit is the length of the i-th array.
    #         Then, scan all the numbers from one to L_1 x L_2 ...
    #         should give all the possible permutaitions. Where L_i is the 
    #         length of the i-th array.
    #         each mumber is decomposed to the numberical system and each digit
    #         represents the item number to take. Thus all the permuatitons are 
    #         achieved.


    my @array_with_references_of_arrays_to_permute = @_;
            #This holds references to n arrays.
    my @lengths;
            #The "base" of the numberical system
    my @permutations;
            #The reporeted results. AKA - Output


    # Createation of the "base":
    foreach my $r_array (@array_with_references_of_arrays_to_permute) {
        my $l = scalar @$r_array;
        push @lengths , $l;
    }

    # Getting all iteration to scan. Basically, L_1 x L_2 ...
    my $number_of_iterations = multiply_list(@lengths);
    $number_of_iterations--;
           #start at 0 (and end up one before).


    # Scan all permutations
    foreach my $iter (0..$number_of_iterations) {
            #iter denoates the iter-th permutation
        my @result_of_iter;
            #holds the items from each array. see at the end

        #the main thing. Decompose the iteration to its 'digits'.
        #unlike normal numberical systems the first item is the LSB,
        #and the last is the MSB. For instance, converting iter=2 in binary will
        #give digits=(0,1) and not digits=(1,0). However, this is synchronized 
        #with the order of the bases (the first array will be scanned first), 
        #so nothing to warry about.
        my @digits = decompose($iter,\@lengths);


        #this loop takes each item from every array. The digits represents which item to take
        #The digit location represents the array. Therefore: [1,0] means take the item in place 1 
        #the first array, and the item in place zero from the second array
        foreach my $i (0..scalar (@digits) -1) {
            my $digit = $digits[$i];
                #the item of the i-th array is in the digit-th place.
            my $item = $array_with_references_of_arrays_to_permute[$i]->[$digit];
                #grap this item.
            push @result_of_iter , $item;
                #store this item with items from the other arrays. 
                # congratulation! you created a new permutaition.
        }

        #print "$_\t" foreach @result_of_iter;
        #print "\n";
        push @permutations , \@result_of_iter;
               #store that permutation.
    }

    return @permutations;
}

sub multiply_list {
    #get a list and return the multiplicity of all items
    my @list = @_;
    my $r = 1;
    foreach my $element (@list) {
        $r *= $element;
    }
    return $r;
}


sub decompose {
    #take a number and a base system. 
    #Decompose the number to digits according to the base system
    #(I found this method for binary system on the internet and
    #converted it to any system).

    my ($t,$r_base) = @_;
    my @results;
    foreach my $base (@$r_base) {
        my $r = $t % $base;
        $t = floor ($t / $base);
        push @results , $r;
    }

    #these are the digits of the new system
    return @results;

}

1;
